unit SRColBtn;

{ TSRColorButton (C)opyright 1999 Version 1.00
  Autor : Simon Reinhardt
  eMail : S.Reinhardt@WTal.de
  Internet : http://pics.wtal.de

  Diese Komponente ist abgeleitet von TCustomLabel und sie
  ist Public Domain, das Copyright liegt aber beim Autor.}

interface

uses
  {$IFDEF WIN32} Windows, {$ELSE} WinTypes, WinProcs, {$ENDIF}
  SysUtils, Messages, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, ExtCtrls;

const
  VNr = 'v1.0';
  URL = 'http://pics.webset.de';
  DefaultWidth  = 75;
  DefaultHeight = 25;

type
  TNumGlyphs = 0..4;
  TButtonLayout = (blGlyphBottom, blGlyphLeft, blGlyphRight, blGlyphTop);

  TSRColorButton = class(TGraphicControl)
  private
    FBevelContrast:    integer;
    FBevelWidth:       integer;
    FBorderStyle:      TBorderStyle;
    FColor:            TColor;
    FColorHighlight:   TColor;
    FColorShadow:      TColor;
    FDown:             boolean;
    FFont:             TFont;
    FGlyph:            TBitmap;
    FLayout:           TButtonLayout;
    FMargin:           integer;
    FNumGlyphs:        TNumGlyphs;
    FSpacing:          integer;

    FMouseDown:        boolean;
    FOnClick:          TNotifyEvent;

    procedure CMEnabledChanged(var Message:TMessage); message CM_ENABLEDCHANGED;

  protected
    procedure DrawBorder;
    procedure Paint;  override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure SetBevelContrast(newValue: integer);
    procedure SetBevelWidth(newValue: integer);
    procedure SetBorderStyle(newBorderStyle: TBorderStyle);
    procedure SetColor(newColor: TColor);
    procedure SetFont(newFont: TFont);
    procedure SetGlyph(newGlyph: TBitmap);
    procedure SetLayout(newValue: TButtonLayout);
    procedure SetMargin(Value: integer);
    procedure SetNumGlyphs(newNumGlyphs: TNumGlyphs);
    procedure SetSpacing(Value: integer);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure CMTextChanged(var msg: TMessage);message CM_TEXTCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar);message CM_DIALOGCHAR;

  published
    property BevelContrast: integer read FBevelContrast write SetBevelContrast;
    property BevelWidth: integer read FBevelWidth write SetBevelWidth;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle;
    property Caption;
    property Color: TColor read FColor write SetColor;
    property Enabled;
    property Font: TFont read FFont write SetFont;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property Layout: TButtonLayout read FLayout write SetLayout;
    property Margin: integer read FMargin write SetMargin;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs default 0;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Spacing: integer read FSpacing write SetSpacing;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

{$IFDEF WIN32}
{$R *.D32}
{$ELSE}
{$R *.D16}
{$ENDIF}

function IsAccellerator(VK: Word; const Str: string): Boolean;
var
  P : Integer;
begin
  P := Pos('&', Str);
  Result := (P <> 0) and (P < Length(Str)) and
    (Upcase(Str[P + 1])=Upcase(Char(VK)));
end;

function ChangeBrightness(Color:TColor;Percentage:longint):TColor;
var RGBColor       : longint;
    Red,Green,Blue : byte;
    NewR,NewG,NewB : longint;
    Overflow       : longint;
begin
  RGBColor:=ColorToRGB(Color);
  Overflow:=0;
  {Rot}
  Red:=GetRValue(RGBColor);
  NewR:=Red+(Percentage*Red div 100);
  if NewR>255 then begin
    Overflow:=NewR-255;
    NewG:=Overflow;
    NewB:=Overflow;
  end
  else begin
    NewG:=0;
    NewB:=0;
  end;
  {Grn}
  Green:=GetGValue(RGBColor);
  NewG:=NewG+Green+(Percentage*Green div 100);
  if NewG>255 then begin
    Overflow:=NewG-255;
    NewR:=NewR+Overflow;
    NewB:=Overflow;
  end;
  {Blau}
  Blue:=GetBValue(RGBColor);
  NewB:=NewB+Blue+(Percentage*Blue div 100);
  if NewB>255 then begin
    Overflow:=NewB-255;
    if NewG<=255 then
      NewR:=NewR+Overflow;
  end;
  if NewR>255 then
    NewR:=255;
  if NewG>255 then
    NewG:=255;
  if NewB>255 then
    NewB:=255;
  if NewR<0 then
    NewR:=0;
  if NewG<0 then
    NewG:=0;
  if NewB<0 then
    NewB:=0;
  Result:=NewR+(NewG shl 8)+(NewB shl 16);
end;


{Komponente SRColorButton}
constructor TSRColorButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  {Vorgabewerte setzen}
  FBevelContrast:=50;
  FBevelWidth:=1;
  FBorderStyle:=bsSingle;
  FColor:=clBtnFace;
  FColorHighlight:=ChangeBrightness(FColor, FBevelContrast);
  FColorShadow:=ChangeBrightness(FColor, -FBevelContrast);
  FDown:=false;
  FFont:=TFont.Create;
  FGlyph:=TBitmap.Create;
  FLayout:=blGlyphLeft;
  FMargin:=1;
  FNumGlyphs:=0;
  FSpacing:=1;
  Height:=25;
  Width:=75;

  FMouseDown:=False;
end;

destructor  TSRColorButton.Destroy;
begin
  FFont.Free;
  FGlyph.Free;
  inherited Destroy;
end;

procedure TSRColorButton.SetBevelContrast(newValue: integer);
begin
  if (FBevelContrast<>NewValue) and (NewValue>10) and (NewValue<90) then begin
    FBevelContrast:=NewValue;
    FColorHighlight:=ChangeBrightness(FColor, FBevelContrast);
    FColorShadow:=ChangeBrightness(FColor, -FBevelContrast);
    Invalidate;
  end;
end;

procedure TSRColorButton.SetBevelWidth(NewValue: integer);
begin
  if (FBevelWidth<>NewValue) and (NewValue>=0) and (NewValue<(Height div 2)) and (NewValue<(Width div 2)) then begin
    FBevelWidth:=NewValue;
    Invalidate;
  end;
end;

procedure TSRColorButton.SetBorderStyle(NewBorderStyle: TBorderStyle);
begin
  if FBorderStyle<>NewBorderStyle then begin
    FBorderStyle:=NewBorderStyle;
    Invalidate;
  end;
end;

procedure TSRColorButton.SetColor(newColor: TColor);
begin
  if FColor<>newColor then begin
    FColor:=newColor;
    FColorHighlight:=ChangeBrightness(FColor,50);
    FColorShadow:=ChangeBrightness(FColor,-50);
    Invalidate;
  end;
end;

procedure TSRColorButton.SetFont(newFont: TFont);
begin
  if FFont<>newFont then begin
    FFont.Assign(NewFont);
    Invalidate;
  end;
end;

procedure TSRColorButton.SetGlyph(newGlyph: TBitmap);
begin
  if(Assigned(FGlyph)) then begin
    FGlyph.Assign(newGlyph);

    if (csDesigning in ComponentState) then begin
      { Glyph 1: Normal, 2: Disabled, 3: Down;
        Mu die Ausmae (Height * NumGlyphs) = Width  haben}
      if (newGlyph.width mod newGlyph.height = 0) then
        FNumGlyphs:= newGlyph.width div newGlyph.height
      else
        FNumGlyphs:= 1;
    end;

    Invalidate;
  end;
end;

procedure TSRColorButton.SetLayout(newValue: TButtonLayout);
begin
  if FLayout<>newValue then begin
    FLayout:=newValue;
    Invalidate;
  end;
end;

procedure TSRColorButton.SetMargin(Value: integer);
begin
  if FMargin<>Value then begin
    FMargin:=Value;
    Invalidate;
  end;
end;

procedure TSRColorButton.SetNumGlyphs(newNumGlyphs: TNumGlyphs);
begin
  if FNumGlyphs<>newNumGlyphs then begin
    FNumGlyphs:= newNumGlyphs;
    Invalidate;
  end;
end;

procedure TSRColorButton.SetSpacing(Value: integer);
begin
  if FSpacing<>Value then begin
    FSpacing:=Value;
    Invalidate;
  end;
end;

procedure TSRColorButton.DrawBorder;
var i    : integer;
    Dest : TRect;
begin
  Dest:=GetClientRect;
  with Canvas do begin
    if FBorderStyle=bsSingle then begin
      Brush.Color:=clWindowFrame;
      FrameRect(Dest);
      InflateRect(Dest,-1,-1);
    end;
    Pen.Width:=1;

    {links + oben}
    if FDown then
      Pen.Color:=FColorShadow
    else
      Pen.Color:=FColorHighlight;
    for i:=0 to FBevelWidth-1 do begin
      MoveTo(Dest.Right-i-1, Dest.Top+i);
      LineTo(Dest.Left+i, Dest.Top+i);
      LineTo(Dest.Left+i, Dest.Bottom-i-1);
    end;

    {rechts + unten}
    if FDown then
      Pen.Color:=FColorHighlight
    else
      Pen.Color:=FColorShadow;
    for i:=0 to FBevelWidth-1 do begin
      MoveTo(Dest.Right-i-1, Dest.Top+i);
      LineTo(Dest.Right-i-1, Dest.Bottom-i-1);
      LineTo(Dest.Left+i, Dest.Bottom-i-1);
    end;

  end;
end;

procedure TSRColorButton.Paint;
var
  Dest,Source,
  CRect,TextR        : TRect;
  CWidth,CHeight,
  outWidth,outHeight,
  TextLeft,TextTop   : integer;
  outText            : array [0..79] of char;
begin
  CRect:=GetClientRect;
  if FBorderStyle=bsSingle then
    InflateRect(CRect, -1, -1);
  InflateRect(CRect,-FBevelWidth,-FBevelWidth);
  CWidth:=CRect.Right-CRect.Left;
  CHeight:=CRect.Bottom-CRect.Top;
  Canvas.Font.Assign(FFont);

  with Canvas do begin
    Brush.Style:= bsSolid;
    Brush.Color:= FColor;
    FillRect(CRect);
  end;

  {Glyph zeichnen ?}
  if Assigned(FGlyph) then begin
     with Source do begin
       { Source-Rechteck ermitteln }
       Left:=0;
       Right:=FGlyph.Width;
       Top:=0;
       Bottom:=FGlyph.Height;
       if FNumGlyphs>0 then
         Right:=Right div FNumGlyphs;
     end;
  end;

  with Canvas do begin
    { Glyph anzeigen }
    outWidth:=  0;
    outHeight:= 0;
    if Assigned(FGlyph) and (FNumGlyphs > 0) then begin
      if(Not Enabled and (FNumGlyphs > 1)) then begin
        { disabled button }
        Source.Left:=  FGlyph.width div FNumGlyphs;
        Source.Right:= Source.Left shl 1;
      end;
      { Gre des Destination-Rechtecks }
      outWidth:=  Source.Right-Source.Left;
      outHeight:= Source.Bottom-Source.Top;
      { Glyph-Position ermitteln }
      if (Caption='') or (FLayout=blGlyphTop) or (FLayout=blGlyphBottom) then begin
        Dest.Left:=  ((CRect.Right-outWidth) shr 1);
        Dest.Right:= Dest.Left+outWidth;
      end;
      if (Caption<>'') and (FLayout=blGlyphLeft) then begin
        Dest.Left:=  ((CRect.Right-(outWidth+FSpacing+TextWidth(Caption))) shr 1)-FMargin;
        Dest.Right:= Dest.Left+outWidth;
      end;
      if (Caption<>'') and (FLayout=blGlyphRight) then begin
        Dest.Left:=  ((CRect.Right+(outWidth+FSpacing+TextWidth(Caption))) shr 1)-outWidth+FMargin;
        Dest.Right:= Dest.Left+outWidth;
      end;
      if (Caption='') or (FLayout=blGlyphLeft) or (FLayout=blGlyphRight) then begin
        Dest.Top:=   ((CRect.Bottom-outHeight) shr 1);
        Dest.Bottom:=Dest.Top+outHeight;
      end;
      if (Caption<>'') and (FLayout=blGlyphTop) then begin
        Dest.Top:=  ((CRect.Bottom-(outHeight+FSpacing+TextHeight(Caption))) shr 1)-FMargin;
        Dest.Bottom:= Dest.Top+outHeight;
      end;
      if (Caption<>'') and (FLayout=blGlyphBottom) then begin
        Dest.Top:=  ((CRect.Bottom-(outHeight+FSpacing+TextHeight(Caption))) shr 1)-outHeight+FMargin;
        Dest.Bottom:= Dest.Top+outHeight;
      end;
      Pen.Style := psSolid;
      Pen.Color := Color;
      if FDown then begin
        { Glyph um 1 Pixel nach rechts unten verschieben }
        Inc(Dest.Left);
        Inc(Dest.Right);
        Inc(Dest.Top);
        Inc(Dest.Bottom);
        { verbleibende Up-Reste lschen }
        MoveTo(Dest.Left-1, Dest.Bottom);
        LineTo(Dest.Left-1, Dest.Top-1);
        LineTo(Dest.Right, Dest.Top-1);
      end
      else begin
        { verbleibende Down-Reste lschen }
        MoveTo(Dest.Right, Dest.Top);
        LineTo(Dest.Right, Dest.Bottom);
        LineTo(Dest.Left, Dest.Bottom);
      end;
      if (FDown and (FNumGlyphs > 2)) then begin
        { Glyph fr gedrckten Zustand bestimmen }
        Source.Left:= FGlyph.width div FNumGlyphs * 2;
        Source.Right:=FGlyph.width div FNumGlyphs * 3;
      end;
      Brush.Style:= bsSolid;
      Brush.Color:= Color;

      { Glyph zeichnen }
      BrushCopy(Dest, FGlyph, Source, FGlyph.Canvas.Pixels[0,FGlyph.Height-1]);
    end;

    { Caption zeichnen }
    if Caption<>'' then begin
      {Position ermitteln}
      TextLeft:=(CRect.Right-TextWidth(Caption)) div 2;
      if Assigned(FGlyph) and (FNumGlyphs > 0) and (FLayout=blGlyphRight) then
        TextLeft:=Dest.Left-TextWidth(Caption)-FSpacing;
      if Assigned(FGlyph) and (FNumGlyphs > 0) and (FLayout=blGlyphLeft) then
        TextLeft:=Dest.Left+outWidth+FSpacing;
      TextTop:=(CRect.Bottom-TextHeight(Caption)) div 2;
      if Assigned(FGlyph) and (FNumGlyphs > 0) and (FLayout=blGlyphTop) then
        TextTop:=Dest.Top+outHeight+FSpacing;
      if Assigned(FGlyph) and (FNumGlyphs > 0) and (FLayout=blGlyphBottom) then
        TextTop:=Dest.Top-TextHeight(Caption)-FSpacing;
      if FDown then
        inc(TextTop);
      {Text ausgeben}
      Brush.Style := bsSolid;
      Brush.Color := Color;
      if FDown then
        { verbleibende Up-Reste lschen }
        FillRect(Rect( TextLeft, TextTop, TextLeft+TextWidth(Caption), TextTop+TextHeight(Caption)))
      else
        { verbleibende Down-Reste lschen }
        FillRect(Rect( TextLeft+1, TextTop+1, TextLeft+1+TextWidth(Caption), TextTop+1+TextHeight(Caption)));
      TextR:=Rect( TextLeft, TextTop, TextLeft+TextWidth(Caption), TextTop+TextHeight(Caption));
      StrPCopy(outText, Caption);
      DrawText(Handle, outText, length(Caption), TextR, DT_Center or DT_VCenter or DT_SingleLine);
    end;
  end;
  DrawBorder;
end;

procedure TSRColorButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do begin
    if IsAccellerator(CharCode, Caption) then begin
      if Enabled then begin
        Click;
        Invalidate;
      end;
      Result := 1;
    end
    else
      inherited;
  end;
end;

procedure TSRColorButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if Enabled then begin
    FDown:=true;
    DrawBorder;
  end;
  FMouseDown:= True;
end;

procedure TSRColorButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  FDown:=false;
  DrawBorder;
  if Enabled then
    if Assigned(FOnClick) then
       FOnClick(Self);
  FMouseDown:= False;
end;

procedure TSRColorButton.CMEnabledChanged(var Message:TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TSRColorButton.CMTextChanged(var msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('Simon',[TSRColorButton]);
end;

end.
